Attribute VB_Name = "basGraphical"
Option Explicit

Private Const ArrayBuffer = 128

Public Enum OperationEnum
    opNone
    opCompare
    opExchange
    opOverwrite
    opSilentExchange
    opMerge
End Enum

Public Type OperationType
    Operation As OperationEnum
    Swap1 As Long
    Swap2 As Long
    Anchor As Long
    Value As Long
End Type

Private mlngArray() As Long
Private mtypOp() As OperationType
Private mlngOps As Long
Private mlngComparisons As Long
Private mlngExchanges As Long
Private mlngArraySize As Long

Private Sub Compare(plng1 As Long, plng2 As Long)
    mlngComparisons = mlngComparisons + 1
    LogOperation opCompare, plng1, plng2, 0, 0
End Sub

Private Sub Exchange(plng1 As Long, plng2 As Long)
    Dim lngSwap As Long
    
    lngSwap = mlngArray(plng1)
    mlngArray(plng1) = mlngArray(plng2)
    mlngArray(plng2) = lngSwap
    mlngExchanges = mlngExchanges + 2  ' Will get divided by two for display
    ' Exchanges take twice as long, so log an extra comparison
    LogOperation opCompare, plng1, plng2, 0, 0
    LogOperation opExchange, plng1, plng2, 0, 0
End Sub

Public Function GraphicalSort(penAlgorithm As AlgorithmEnum, plngArray() As Long, ptypOp() As OperationType, plngCompare As Long, plngExchange As Long) As Long
    Erase mtypOp
    mlngArraySize = 0
    mlngOps = 0
    mlngComparisons = 0
    mlngExchanges = 0
    mlngArray = plngArray
    Select Case penAlgorithm
        Case aeBubblesort: BubbleSort mlngArray
        Case aeCocktailsort: CocktailSort mlngArray
        Case aeCombsort: CombSort mlngArray
'        Case aeCountingSort: CountingSort mlngArray
        Case aeGnomesort: GnomeSort mlngArray
        Case aeHeapsort: HeapSort mlngArray
        Case aeInsertionsort: InsertionSort mlngArray
        Case aeJSort: JSort mlngArray
        Case aeJumpsort: JumpSort mlngArray
        Case aeLinkedListSort: LinkedListSort mlngArray
        Case aeMedianThreeQuicksort: MedianThreeQuickSort mlngArray
        Case aeMergesort: MergeSort mlngArray
        Case aeQuicksort: QuickSort mlngArray
        Case aeSelectionsort: SelectionSort mlngArray
        Case aeShakersort: ShakerSort mlngArray
        Case aeShearsort: ShearSort mlngArray
        Case aeShellsort: ShellSort mlngArray
        Case aeSmoothsort: SmoothSort mlngArray
        Case aeSnakesort: SnakeSort mlngArray
    End Select
    ptypOp = mtypOp
    plngCompare = mlngComparisons
    plngExchange = mlngExchanges
    Erase mlngArray, mtypOp
End Function

Private Sub LogOperation(penOperation As OperationEnum, plng1 As Long, plng2 As Long, plngAnchor As Long, plngValue As Long)
    If mlngArraySize = 0 Then
        ReDim mtypOp(ArrayBuffer)
        mlngArraySize = ArrayBuffer
    ElseIf mlngOps >= mlngArraySize Then
        mlngArraySize = mlngArraySize + ArrayBuffer
        ReDim Preserve mtypOp(mlngArraySize)
    End If
    With mtypOp(mlngOps)
        .Operation = penOperation
        .Swap1 = plng1
        .Swap2 = plng2
        .Anchor = plngAnchor
        .Value = plngValue
    End With
    mlngOps = mlngOps + 1
End Sub

Private Sub Merge(plngIndex As Long, ByVal plngValue As Long, plngLeft As Long, plngRight As Long)
    mlngExchanges = mlngExchanges + 1
    mlngArray(plngIndex) = plngValue
    LogOperation opMerge, plngLeft, plngRight, plngIndex, plngValue
End Sub

Public Sub OrderArray(plngArray() As Long, ByVal penOrder As OrderEnum, Optional plngPercent As Long = 5)
    Dim i As Long
    Dim iMin As Long
    Dim iMax As Long
    Dim lngMid As Long
    Dim lngIndex As Long
    Dim lngSwap As Long
    Dim lngMirror() As Long
    Dim lngLeft As Long
    Dim lngRight As Long
    Dim lngStep As Long
    
    iMin = LBound(plngArray)
    iMax = UBound(plngArray)
    lngMid = (iMax - iMin) \ 2
    Select Case penOrder
        Case oeAscending
            ShellSort1 plngArray
        Case oeDescending
            ShellSort1 plngArray
            For i = 0 To (iMax - iMin) \ 2
                lngSwap = plngArray(iMin + i)
                plngArray(iMin + i) = plngArray(iMax - i)
                plngArray(iMax - i) = lngSwap
            Next
        Case oeRandom
            ShuffleArray plngArray
        Case oeWeave
            ShellSort1 plngArray
'            WeaveArray plngArray, iMin, iMax
            For i = iMin To iMax \ 2
                plngArray(i) = i
            Next
            lngIndex = i
            For i = lngIndex To iMax
                plngArray(i) = i - lngIndex
            Next
        Case oeCamel
            ShellSort1 plngArray
            ReDim lngMirror(iMin To iMax)
            If (iMax - iMin) Mod 2 = 0 Then
                lngIndex = iMin
                For i = iMin To iMax Step 2
                    lngMirror(lngIndex) = plngArray(i)
                    lngIndex = lngIndex + 1
                Next
                lngIndex = iMax
                For i = iMin + 1 To iMax Step 2
                    lngMirror(lngIndex) = plngArray(i)
                    lngIndex = lngIndex - 1
                Next
            Else
                lngIndex = iMin
                For i = iMin To iMax Step 2
                    lngMirror(lngIndex) = plngArray(i)
                    lngIndex = lngIndex + 1
                Next
                lngIndex = iMax
                For i = iMin + 1 To iMax Step 2
                    lngMirror(lngIndex) = plngArray(i)
                    lngIndex = lngIndex - 1
                Next
            End If
            plngArray = lngMirror
            Erase lngMirror
        Case oePercent
            ShellSort1 plngArray
            lngStep = (iMax - iMin + 1) * plngPercent / 100
            If lngStep > 1 Then
                lngLeft = iMin
                Do
                    lngRight = lngLeft + lngStep - 1
                    If lngRight > iMax Then lngRight = iMax
                    For i = lngRight To lngLeft + 1 Step -1
                        lngMid = Int((i - lngLeft + 1) * Rnd + lngLeft)
                        lngSwap = plngArray(i)
                        plngArray(i) = plngArray(lngMid)
                        plngArray(lngMid) = lngSwap
                    Next
                    lngLeft = lngLeft + lngStep
                Loop Until lngLeft >= iMax
            End If
'        Case oeThatched
'            ShellSort1 plngArray
'            ReDim lngMirror(iMin To iMax)
'            If (iMax - iMin) Mod 2 = 0 Then
'                For i = iMin + 1 To (iMax - iMin) Step 2
'                    lngMirror(i - 1) = plngArray(i - 1)
'                    lngMirror(i) = plngArray(iMax - i)
'                Next
'                lngMirror(iMax) = plngArray(iMax)
'            Else
'                For i = iMin To (iMax - iMin) Step 2
'                    lngMirror(i) = plngArray(i)
'                    lngMirror(i + 1) = plngArray(iMax - i)
'                Next
'            End If
'            plngArray = lngMirror
'            Erase lngMirror
'        Case oeConcave
'            ShellSort1 plngArray
'            ReDim lngMirror(iMin To iMax)
'            If (iMax - iMin) Mod 2 = 0 Then
'                lngIndex = lngMid
'                For i = iMin To iMax Step 2
'                    lngMirror(lngIndex) = plngArray(i)
'                    lngIndex = lngIndex + 1
'                Next
'                lngIndex = lngMid - 1
'                For i = iMin + 1 To iMax Step 2
'                    lngMirror(lngIndex) = plngArray(i)
'                    lngIndex = lngIndex - 1
'                Next
'            Else
'                lngIndex = lngMid + 1
'                For i = iMin To iMax Step 2
'                    lngMirror(lngIndex) = plngArray(i)
'                    lngIndex = lngIndex + 1
'                Next
'                lngIndex = lngMid
'                For i = iMin + 1 To iMax Step 2
'                    lngMirror(lngIndex) = plngArray(i)
'                    lngIndex = lngIndex - 1
'                Next
'            End If
'            plngArray = lngMirror
'            Erase lngMirror
    End Select
End Sub

Private Sub WeaveArray(plngArray() As Long, plngLeft As Long, plngRight As Long)
    Dim lngMirror() As Long
    Dim i As Long
    Dim L As Long
    Dim R As Long
    Dim lngMid As Long
    Dim blnLeft As Boolean
    
    ReDim lngMirror(plngLeft To plngRight)
    lngMid = (plngRight - plngLeft) \ 2 + plngLeft
    If (plngRight - plngLeft) Mod 2 <> 0 Then lngMid = lngMid + 1
    L = plngLeft
    R = lngMid
    For i = plngLeft To plngRight
        If blnLeft Then
            lngMirror(L) = plngArray(i)
            L = L + 1
        Else
            lngMirror(R) = plngArray(i)
            R = R + 1
        End If
        blnLeft = Not blnLeft
    Next
    For i = plngLeft To plngRight
        plngArray(i) = lngMirror(i)
    Next
    Erase lngMirror
    If plngRight - plngLeft > 3 Then
        WeaveArray plngArray, plngLeft, lngMid - 1
        WeaveArray plngArray, lngMid, plngRight
    End If
End Sub

Private Sub Overwrite(plngTo As Long, plngFrom As Long, plngAnchor As Long)
    mlngExchanges = mlngExchanges + 1
    mlngArray(plngTo) = mlngArray(plngFrom)
    LogOperation opOverwrite, plngTo, plngFrom, plngAnchor, 0
End Sub

' Knuth shuffle (very fast)
Public Sub ShuffleArray(pvarArray As Variant)
    Dim i As Long
    Dim iMin As Long
    Dim iMax As Long
    Dim lngReplace As Long
    Dim varSwap As Variant
    
    iMin = LBound(pvarArray)
    iMax = UBound(pvarArray)
    For i = iMax To iMin + 1 Step -1
        lngReplace = Int((i - iMin + 1) * Rnd + iMin)
        varSwap = pvarArray(i)
        pvarArray(i) = pvarArray(lngReplace)
        pvarArray(lngReplace) = varSwap
    Next
End Sub

' Final undisplayable exchange after series of Overwrites (Insertion sort, Merge Sort)
Private Sub SilentExchange(plngExchanges As Long)
    Dim i As Long
    
    mlngExchanges = mlngExchanges + plngExchanges
    For i = 1 To plngExchanges
        LogOperation opSilentExchange, 0, 0, 0, 0
    Next
End Sub

Private Sub SnakeMerge(plngIndex As Long, ByVal plngValue As Long, plngLeft As Long, plngRight As Long)
    mlngExchanges = mlngExchanges + 1
    LogOperation opMerge, plngLeft, plngRight, plngIndex, plngValue
End Sub

' ====================================================
' Sorting Algorithms
' ====================================================

Public Sub BinarySort(ByRef plngArray() As Long)
    Dim i As Long
    Dim j As Long
    Dim iMin As Long
    Dim iMax As Long
    Dim lngMin As Long
    Dim lngMid As Long
    Dim lngMax As Long
    Dim lngPrev() As Long
    Dim lngNext() As Long
    Dim lngIndex() As Long
    Dim varMirror() As Long
    Dim lngPrevious As Long
    Dim lngLastSentinel As Long
    Dim lngSentinelSize As Long
    
    iMin = LBound(plngArray)
    iMax = UBound(plngArray)
    ReDim lngPrev(iMin To iMax)
    ReDim lngNext(iMin To iMax)
    lngSentinelSize = (iMax - iMin + 1) \ 100
    If lngSentinelSize < 5 Then lngSentinelSize = 5
    ReDim lngIndex((iMax - iMin + 1) \ lngSentinelSize)
    lngIndex(0) = iMin
    lngNext(iMin) = -1
    lngPrev(iMin) = -1
    For i = iMin + 1 To iMax
        ' Binary search to locate nearest sentinel
        lngMin = 0
        lngMax = lngLastSentinel
        Do While lngMin <= lngMax
            lngMid = (lngMin + lngMax) \ 2
            Compare lngIndex(lngMid), i
            If plngArray(lngIndex(lngMid)) > plngArray(i) Then
                lngMax = lngMid - 1
            ElseIf plngArray(lngIndex(lngMid)) < plngArray(i) Then
                Compare lngIndex(lngMid), i
                lngMin = lngMid + 1
            Else
                Compare lngIndex(lngMid), i
                Exit Do
            End If
        Loop
        ' Back up to first sentinel <= current item
        Compare lngIndex(lngMid), i
        Do While plngArray(lngIndex(lngMid)) > plngArray(i) And lngMid > 0
            Compare lngIndex(lngMid), i
            lngMid = lngMid - 1
        Loop
        ' Pull subsequent sentinels back  one
        For j = lngMid + 1 To lngLastSentinel
            lngIndex(j) = lngPrev(lngIndex(j))
        Next
        ' Begin searching within the sentinel block
        j = lngIndex(lngMid)
        Do
            Compare i, j
            If plngArray(i) > plngArray(j) Then
                lngPrevious = j
                j = lngNext(j)
                If j = -1 Then
                    lngNext(lngPrevious) = i
                    lngPrev(i) = lngPrevious
                    lngNext(i) = -1
                    Exit Do
                End If
            Else
                lngNext(i) = j
                lngPrev(j) = i
                If j = lngIndex(lngMid) Then
                    lngIndex(lngMid) = i
                Else
                    lngPrev(i) = lngPrevious
                    lngNext(lngPrevious) = i
                End If
                Exit Do
            End If
        Loop
        ' Add new sentinel
        If i Mod lngSentinelSize = 0 Then
            lngMid = lngIndex(lngLastSentinel)
            For j = 1 To lngSentinelSize
                If lngNext(lngMid) = -1 Then Exit For
                lngMid = lngNext(lngMid)
            Next
            lngLastSentinel = lngLastSentinel + 1
            lngIndex(lngLastSentinel) = lngMid
        End If
    Next
    ReDim varMirror(iMin To iMax)
    i = iMin
    j = lngIndex(0)
    Do
        Compare i, j
        varMirror(i) = plngArray(j)
        i = i + 1
        j = lngNext(j)
    Loop Until j = -1
    For i = iMin To iMax
        SnakeMerge i, varMirror(i), iMin, iMax
        plngArray(i) = varMirror(i)
    Next
    Erase lngPrev, lngNext, varMirror, lngIndex
End Sub

Private Sub BubbleSort(ByRef plngArray() As Long)
    Dim i As Long
    Dim iMin As Long
    Dim iMax As Long
    Dim blnSwapped As Boolean
    
    iMin = LBound(plngArray)
    iMax = UBound(plngArray) - 1
    Do
        blnSwapped = False
        For i = iMin To iMax
            Compare i, i + 1
            If plngArray(i) > plngArray(i + 1) Then
                Exchange i, i + 1
                blnSwapped = True
            End If
        Next
        iMax = iMax - 1
    Loop Until Not blnSwapped
End Sub

Private Sub CocktailSort(ByRef plngArray() As Long)
    Dim i As Long
    Dim iMin As Long
    Dim iMax As Long
    Dim blnSwapped As Boolean
    
    iMin = LBound(plngArray)
    iMax = UBound(plngArray) - 1
    Do
        blnSwapped = False
        For i = iMin To iMax
            Compare i, i + 1
            If plngArray(i) > plngArray(i + 1) Then
                Exchange i, i + 1
                blnSwapped = True
            End If
        Next
        iMax = iMax - 1
        If Not blnSwapped Then Exit Do
        For i = iMax To iMin Step -1
            Compare i, i + 1
            If plngArray(i) > plngArray(i + 1) Then
                Exchange i, i + 1
                blnSwapped = True
            End If
        Next
        iMin = iMin + 1
    Loop Until Not blnSwapped
End Sub

Private Sub CombSort(ByRef plngArray() As Long)
    Const ShrinkFactor = 1.3
    Dim lngGap As Long
    Dim i As Long
    Dim iMin As Long
    Dim iMax As Long
    Dim blnSwapped As Boolean
    
    iMin = LBound(plngArray)
    iMax = UBound(plngArray)
    lngGap = iMax - iMin + 1
    Do
        If lngGap > 1 Then
            lngGap = Int(lngGap / ShrinkFactor)
            If lngGap = 10 Or lngGap = 9 Then lngGap = 11
        End If
        blnSwapped = False
        For i = iMin To iMax - lngGap
            Compare i, i + lngGap
            If plngArray(i) > plngArray(i + lngGap) Then
                Exchange i, i + lngGap
                blnSwapped = True
            End If
        Next
    Loop Until lngGap = 1 And Not blnSwapped
End Sub

Private Sub CountingSort(ByRef plngArray() As Long)
    Dim i As Long
    Dim j As Long
    Dim k As Long
    Dim iMin As Long
    Dim iMax As Long
    Dim lngLowest As Long
    Dim lngHighest As Long
    Dim lngBucket() As Long
    
    iMin = LBound(plngArray)
    iMax = UBound(plngArray)
    ' Get highest and lowest value
    lngLowest = iMin
    lngHighest = iMin
    For i = iMin + 1 To iMax
        Compare i, lngLowest
        If plngArray(i) < plngArray(lngLowest) Then lngLowest = i
        Compare i, lngHighest
        If plngArray(i) > plngArray(lngHighest) Then lngHighest = i
    Next
    SilentExchange 2
    lngLowest = plngArray(lngLowest)
    lngHighest = plngArray(lngHighest)
    ' Count
    ReDim lngBucket(lngLowest To lngHighest)
    For i = iMin To iMax
        Compare i, i
        lngBucket(plngArray(i)) = lngBucket(plngArray(i)) + 1
    Next
    ' Save
    i = iMin
    For j = lngLowest To lngHighest
        For k = 1 To lngBucket(j)
            SnakeMerge i, j, i, i
            i = i + 1
        Next
    Next
    Erase lngBucket
End Sub

Private Sub GnomeSort(ByRef plngArray() As Long)
    Dim i As Long
    Dim j As Long
    Dim iMin As Long
    Dim iMax As Long
    
    iMin = LBound(plngArray) + 1
    iMax = UBound(plngArray)
    i = iMin
    j = i + 1
    Do While i <= iMax
        Compare i, i - 1
        If plngArray(i) < plngArray(i - 1) Then
            Exchange i, i - 1
            If i > iMin Then i = i - 1
        Else
            i = j
            j = j + 1
        End If
    Loop
End Sub

Private Sub HeapSort(ByRef plngArray() As Long)
    Dim i As Long
    Dim iMin As Long
    Dim iMax As Long
    
    iMin = LBound(plngArray)
    iMax = UBound(plngArray)
    For i = (iMax + iMin) \ 2 To iMin Step -1
        Heap plngArray, i, iMin, iMax
    Next i
    For i = iMax To iMin + 1 Step -1
        Exchange i, iMin
        Heap plngArray, iMin, iMin, i - 1
    Next i
End Sub

Private Sub Heap(ByRef plngArray() As Long, ByVal i As Long, iMin As Long, iMax As Long)
    Dim lngLeaf As Long
    
    Do
        lngLeaf = i + i - (iMin - 1)
        Select Case lngLeaf
            Case Is > iMax: Exit Do
            Case Is < iMax
                Compare lngLeaf + 1, lngLeaf
                If plngArray(lngLeaf + 1) > plngArray(lngLeaf) Then lngLeaf = lngLeaf + 1
        End Select
        Compare lngLeaf + 1, lngLeaf
        If plngArray(i) > plngArray(lngLeaf) Then Exit Do
        Exchange i, lngLeaf
        i = lngLeaf
    Loop
End Sub

Private Sub InsertionSort(ByRef plngArray() As Long)
    Dim i As Long
    Dim j As Long
    Dim iMin As Long
    Dim iMax As Long
    Dim lngSwap As Long
    
    iMin = LBound(plngArray) + 1
    iMax = UBound(plngArray)
    For i = iMin To iMax
        SilentExchange 1
        lngSwap = plngArray(i)
        For j = i To iMin Step -1
            Compare i, j - 1
            If lngSwap < plngArray(j - 1) Then Overwrite j, j - 1, i Else Exit For
        Next j
        SilentExchange 1
        plngArray(j) = lngSwap
    Next i
End Sub

Public Sub JSort(ByRef plngArray() As Long)
    Dim iMin As Long
    Dim iMax As Long
    Dim i As Long
    Dim j As Long
    Dim lngChild As Long
    Dim lngSwap As Long
    
    iMin = LBound(plngArray)
    iMax = UBound(plngArray)
    For i = iMax To iMin Step -1
        lngChild = 2 * (i + 1) - 1
        If lngChild < iMax Then JSortHeap plngArray, iMax, i, lngChild
    Next i
    For i = iMax To iMin Step -1
        lngChild = 2 * (i + 1) - 1
        If lngChild < iMax Then JSortHeapInv plngArray, iMax, i, lngChild
    Next i
    iMin = iMin + 1
    For i = iMin To iMax
        SilentExchange 1
        lngSwap = plngArray(i)
        For j = i To iMin Step -1
            Compare i, j - 1
            If lngSwap < plngArray(j - 1) Then Overwrite j, j - 1, i Else Exit For
        Next j
        SilentExchange 1
        plngArray(j) = lngSwap
    Next i
End Sub

Private Sub JSortHeap(ByRef plngArray() As Long, ByVal iMax As Long, ByVal plngParent As Long, ByVal plngChild As Long)
    Dim lngHold As Long
    Dim lngCompare As Long
    
    lngCompare = plngParent
    SilentExchange 1
    lngHold = plngArray(plngParent)
    Do While plngChild < iMax
        SilentExchange 1
        If plngArray(plngChild) >= plngArray(plngChild + 1) Then plngChild = plngChild + 1
        Compare lngCompare, plngChild
        If lngHold < plngArray(plngChild) Then Exit Do
        Overwrite plngParent, plngChild, plngParent
        plngParent = plngChild
        plngChild = 2 * (plngParent + 1) - 1
    Loop
    SilentExchange 1
    plngArray(plngParent) = lngHold
End Sub

Private Sub JSortHeapInv(ByRef plngArray() As Long, ByVal iMax As Long, ByVal plngParent As Long, ByVal plngChild As Long)
    Dim lngHold As Long
    Dim lngCompare As Long
    
    lngCompare = iMax - plngParent
    SilentExchange 1
    lngHold = plngArray(iMax - plngParent)
    Do While plngChild < iMax
        SilentExchange 1
        If plngArray(iMax - plngChild) >= plngArray(iMax - plngChild + 1) Then plngChild = plngChild + 1
        Compare lngCompare, iMax - plngChild
        If lngHold > plngArray(iMax - plngChild) Then Exit Do
        Overwrite iMax - plngParent, iMax - plngChild, iMax - plngParent
        plngParent = plngChild
        plngChild = 2 * (plngParent + 1) - 1
    Loop
    SilentExchange 1
    plngArray(iMax - plngParent) = lngHold
End Sub

Private Sub JumpSort(ByRef plngArray() As Long)
    Dim lngJump As Long
    Dim i As Long
    Dim iMin As Long
    Dim iMax As Long
    Dim blnSwapped As Boolean
    
    iMin = LBound(plngArray)
    iMax = UBound(plngArray)
    lngJump = iMax - iMin
    If lngJump < 2 Then lngJump = 2
    Do
        lngJump = lngJump \ 2
        Do
            blnSwapped = False
            For i = iMin To iMax - lngJump
                Compare i, i + lngJump
                If plngArray(i) > plngArray(i + lngJump) Then
                    Exchange i, i + lngJump
                    blnSwapped = True
                End If
            Next
        Loop Until Not blnSwapped
    Loop Until lngJump = 1
End Sub

Public Sub LinkedListSort(ByRef plngArray() As Long)
    Dim i As Long
    Dim j As Long
    Dim iMin As Long
    Dim iMax As Long
    Dim lngFirst As Long
    Dim lngNext() As Long
    Dim lngPrev As Long
    Dim varMirror() As Long
    
    iMin = LBound(plngArray)
    iMax = UBound(plngArray)
    ReDim lngNext(iMin To iMax)
    lngFirst = iMin
    lngNext(iMin) = -1
    For i = iMin + 1 To iMax
        j = lngFirst
        Do
            Compare i, j
            If plngArray(i) > plngArray(j) Then
                lngPrev = j
                j = lngNext(j)
                If j = -1 Then
                    lngNext(lngPrev) = i
                    lngNext(i) = -1
                    Exit Do
                End If
            Else
                lngNext(i) = j
                If j = lngFirst Then lngFirst = i Else lngNext(lngPrev) = i
                Exit Do
            End If
        Loop
    Next
    ReDim varMirror(iMin To iMax)
    i = iMin
    j = lngFirst
    Do
        Compare i, j
        varMirror(i) = plngArray(j)
        i = i + 1
        j = lngNext(j)
    Loop Until j = -1
    For i = iMin To iMax
        SnakeMerge i, varMirror(i), iMin, iMax
        plngArray(i) = varMirror(i)
    Next
    Erase lngNext, varMirror
End Sub

' Omit pvarMirror, plngLeft & plngRight; they are used internally during recursion
Private Sub MergeSort(ByRef plngArray() As Long, Optional pvarMirror As Variant, Optional ByVal plngLeft As Long, Optional ByVal plngRight As Long)
    Dim lngMid As Long
    Dim L As Long
    Dim R As Long
    Dim O As Long
    Dim varSwap As Variant

    If plngRight = 0 Then
        plngLeft = LBound(plngArray)
        plngRight = UBound(plngArray)
        ReDim pvarMirror(plngLeft To plngRight)
    End If
    lngMid = plngRight - plngLeft
    Select Case lngMid
        Case 0
        Case 1
            Compare plngLeft, plngRight
            If plngArray(plngLeft) > plngArray(plngRight) Then
                Exchange plngLeft, plngRight
            End If
        Case Else
            lngMid = lngMid \ 2 + plngLeft
            MergeSort plngArray, pvarMirror, plngLeft, lngMid
            MergeSort plngArray, pvarMirror, lngMid + 1, plngRight
            ' Merge the resulting halves
            L = plngLeft ' start of first (left) half
            R = lngMid + 1 ' start of second (right) half
            O = plngLeft ' start of output (mirror array)
            Do
                Compare L, R
                If plngArray(R) < plngArray(L) Then
                    SilentExchange 1
                    pvarMirror(O) = plngArray(R)
                    R = R + 1
                    If R > plngRight Then
                        For L = L To lngMid
                            O = O + 1
                            SilentExchange 1
                            pvarMirror(O) = plngArray(L)
                        Next
                        Exit Do
                    End If
                Else
                    SilentExchange 1
                    pvarMirror(O) = plngArray(L)
                    L = L + 1
                    If L > lngMid Then
                        For R = R To plngRight
                            O = O + 1
                            SilentExchange 1
                            pvarMirror(O) = plngArray(R)
                        Next
                        Exit Do
                    End If
                End If
                O = O + 1
            Loop
            For O = plngLeft To plngRight
                Merge O, pvarMirror(O), plngLeft, plngRight
            Next
    End Select
End Sub

' Omit plngLeft & plngRight; they are used internally during recursion
Private Sub QuickSort(ByRef plngArray() As Long, Optional ByVal plngLeft As Long, Optional ByVal plngRight As Long)
    Dim lngFirst As Long
    Dim lngLast As Long
    Dim lngMid As Long
    Dim lngIndex As Long
    
    If plngRight = 0 Then
        plngLeft = LBound(plngArray)
        plngRight = UBound(plngArray)
    End If
    lngFirst = plngLeft
    lngLast = plngRight
    lngIndex = (plngLeft + plngRight) \ 2
    lngMid = plngArray(lngIndex)
    Do
        Compare lngIndex, lngFirst
        Do While plngArray(lngFirst) < lngMid And lngFirst < plngRight
            lngFirst = lngFirst + 1
            Compare lngIndex, lngFirst
        Loop
        Compare lngIndex, lngLast
        Do While lngMid < plngArray(lngLast) And lngLast > plngLeft
            Compare lngIndex, lngLast
            lngLast = lngLast - 1
        Loop
        If lngFirst <= lngLast Then
            Exchange lngFirst, lngLast
            lngFirst = lngFirst + 1
            lngLast = lngLast - 1
        End If
    Loop Until lngFirst > lngLast
    If plngLeft < lngLast Then QuickSort plngArray, plngLeft, lngLast
    If lngFirst < plngRight Then QuickSort plngArray, lngFirst, plngRight
End Sub

' Omit plngLeft & plngRight; they are used internally during recursion
Private Sub MedianThreeQuickSort(ByRef plngArray() As Long, Optional ByVal plngLeft As Long, Optional ByVal plngRight As Long)
    Dim lngFirst As Long
    Dim lngLast As Long
    Dim lngMid As Long
    Dim lngIndex As Long
    Dim a As Long
    Dim b As Long
    Dim c As Long
    
    If plngRight = 0 Then
        plngLeft = LBound(plngArray)
        plngRight = UBound(plngArray)
    End If
    lngFirst = plngLeft
    lngLast = plngRight
    lngIndex = plngRight - plngLeft + 1
    a = Int(lngIndex * Rnd) + plngLeft
    b = Int(lngIndex * Rnd) + plngLeft
    c = Int(lngIndex * Rnd) + plngLeft
'    Compare a, b
'    Compare b, c
    If plngArray(a) <= plngArray(b) And plngArray(b) <= plngArray(c) Then
        lngIndex = b
    Else
'        Compare b, a
'        Compare a, c
        If plngArray(b) <= plngArray(a) And plngArray(a) <= plngArray(c) Then
            lngIndex = a
        Else
            lngIndex = c
        End If
    End If
    lngMid = plngArray(lngIndex)
    Do
        Compare lngIndex, lngFirst
        Do While plngArray(lngFirst) < lngMid And lngFirst < plngRight
            lngFirst = lngFirst + 1
            Compare lngIndex, lngFirst
        Loop
        Compare lngIndex, lngLast
        Do While lngMid < plngArray(lngLast) And lngLast > plngLeft
            Compare lngIndex, lngLast
            lngLast = lngLast - 1
        Loop
        If lngFirst <= lngLast Then
            Exchange lngFirst, lngLast
            lngFirst = lngFirst + 1
            lngLast = lngLast - 1
        End If
    Loop Until lngFirst > lngLast
    If plngLeft < lngLast Then QuickSort plngArray, plngLeft, lngLast
    If lngFirst < plngRight Then QuickSort plngArray, lngFirst, plngRight
End Sub

Private Sub SelectionSort(ByRef plngArray() As Long)
    Dim i As Long
    Dim j As Long
    Dim iMin As Long
    Dim iMax As Long
    Dim varSwap As Variant

    iMin = LBound(plngArray)
    iMax = UBound(plngArray)
    For i = iMin To iMax - 1
        iMin = i
        For j = (i + 1) To iMax
            Compare iMin, j
            If plngArray(j) < plngArray(iMin) Then iMin = j
        Next
        Exchange iMin, i
    Next
End Sub

Public Function ShakerSort(ByRef plngArray() As Long)
    Dim i As Long
    Dim j As Long
    Dim k As Long
    Dim iMin As Long
    Dim iMax As Long
    Dim varSwap As Variant
    Dim blnSwapped As Boolean
    
    iMin = LBound(plngArray)
    iMax = UBound(plngArray)
    i = (iMax - iMin) * 1999 \ 2000 + iMin
    Do While i > iMin
        j = i
        blnSwapped = False
        Do While j > iMin
            For k = iMin To iMax - j
                Compare k, k + j
                If plngArray(k) > plngArray(k + j) Then
                    Exchange k, k + j
                    blnSwapped = True
                End If
            Next
            If blnSwapped Then
                j = j * 7 \ 10
            ElseIf j > k \ 2 Then
                j = j \ 2
            Else
                Exit Do
            End If
        Loop
        If i > iMin + 3 Then
            i = i \ 20
        ElseIf i > iMin + 2 Then
            i = iMin + 2
        ElseIf i > iMin + 1 Then
            i = iMin + 1
        Else
            Exit Do
        End If
    Loop
    iMax = iMax - 1
    Do
        blnSwapped = False
        For i = iMin To iMax
            Compare i, i + 1
            If plngArray(i) > plngArray(i + 1) Then
                Exchange i, i + 1
                blnSwapped = True
            End If
        Next i
        If blnSwapped Then
            blnSwapped = False
            iMax = iMax - 1
            For i = iMax To iMin Step -1
               Compare i, i + 1
                If plngArray(i) > plngArray(i + 1) Then
                    Exchange i, i + 1
                    blnSwapped = True
                End If
            Next i
            iMin = iMin + 1
        End If
    Loop Until Not blnSwapped
End Function

Public Function ShearSort(ByRef plngArray() As Long)
    Dim Log As Long, Rows As Long, Cols As Long
    Dim pow As Long, div As Long
    Dim h() As Long
    Dim i As Long, k As Long, j As Long
    
    Dim LMax As Long, LMin As Long
    LMax = UBound(plngArray) + 1
    LMin = LBound(plngArray)
    pow = 1
    div = 1
    Do While i * i <= LMax
        If i > 0 Then
            If LMax Mod i = 0 Then div = i
        Else
            div = i
        End If
        i = i + 1
    Loop
    Rows = div
    Cols = LMax \ div
    Do While pow <= Rows
        pow = pow * 2
        Log = Log + 1
    Loop
    ReDim h(Rows)
    For i = 0 To Rows
        h(i) = i * Cols
    Next i
    
    For k = 0 To Log - 1
        For j = 0 To Cols \ 2 - 1
            For i = 0 To Rows - 1
                ShearPart1 plngArray, h(i), h(i + 1), 1, i Mod 2 = 0
            Next i
            For i = 0 To Rows - 1
                ShearPart2 plngArray, h(i), h(i + 1), 1, i Mod 2 = 0
            Next i
        Next j
        For j = 0 To Rows \ 2 - 1
            For i = 0 To Cols - 1
                ShearPart1 plngArray, i, Rows * Cols + i, Cols, True
            Next i
            For i = 0 To Cols - 1
                ShearPart2 plngArray, i, Rows * Cols + i, Cols, True
            Next i
        Next j
    Next k

    For j = 0 To Cols \ 2 - 1
        For i = 0 To Rows - 1
            ShearPart1 plngArray, h(i), h(i + 1), 1, True
        Next i
        For i = 0 To Rows - 1
            ShearPart2 plngArray, h(i), h(i + 1), 1, True
        Next i
    Next j

    For i = 0 To Rows - 1
        h(i) = -1
    Next i
    
    GnomeSort plngArray ' Because I'm too lazy to debug the algorithm
End Function

Private Sub ShearPart1(ByRef a() As Long, ByVal Lo As Long, ByVal Hi As Long, ByVal Nx As Long, ByVal Up As Boolean)
    Dim t As Long
    Dim j As Long
    j = Lo
    If Up Then
        Do While j + Nx < Hi
            Compare j, j + Nx
            If a(j) > a(j + Nx) Then
                Exchange j, j + Nx
            End If
            j = j + 2 * Nx
        Loop
    Else
        Do While j + Nx < Hi
            Compare j, j + Nx
            If a(j) < a(j + Nx) Then
                Exchange j, j + Nx
            End If
            j = j + 2 * Nx
        Loop
    End If
End Sub

Private Sub ShearPart2(ByRef a As Variant, ByVal Lo As Long, ByVal Hi As Long, ByVal Nx As Long, ByVal Up As Boolean)
    Dim t As Variant
    Dim j As Long
    j = Lo + Nx
    If Up Then
        Do While j + Nx < Hi
            Compare j, j + Nx
            If a(j) > a(j + Nx) Then
                Exchange j, j + Nx
            End If
            j = j + 2 * Nx
        Loop
    Else
        Do While j + Nx < Hi
            Compare j, j + Nx
            If a(j) < a(j + Nx) Then
                Exchange j, j + Nx
            End If
            j = j + 2 * Nx
        Loop
    End If
End Sub

Private Sub ShellSort(ByRef plngArray() As Long)
    Dim lngHold As Long
    Dim lngGap As Long
    Dim i As Long
    Dim iMin As Long
    Dim iMax As Long
    Dim lngSwap As Long
    Dim lngElements As Long
    Dim lngMin As Long
    
    iMin = LBound(plngArray)
    iMax = UBound(plngArray)
    lngElements = iMax - iMin + 1
    Do
        lngGap = 3 * lngGap + 1
    Loop Until lngGap > lngElements
    Do
        lngGap = lngGap \ 3
        lngMin = iMin + lngGap
        For i = lngMin To iMax
            SilentExchange 1
            lngSwap = plngArray(i)
            lngHold = i
            Compare i, lngHold - lngGap
            Do While plngArray(lngHold - lngGap) > lngSwap
                Overwrite lngHold, lngHold - lngGap, i
                lngHold = lngHold - lngGap
                If lngHold < lngMin Then Exit Do
            Loop
            SilentExchange 1
            plngArray(lngHold) = lngSwap
        Next i
    Loop Until lngGap = 1
End Sub

' Original algorithm I created while writing this
' project. This rivals quicksort speed on randomly
' ordered lists, but is much faster on lists that are
' already ordered to any extent.
Private Sub SnakeSort(ByRef plngArray() As Long)
    Dim i As Long
    Dim iMin As Long
    Dim iMax As Long
    Dim lngIndex() As Long
    Dim lngLevel As Long
    Dim lngOldLevel As Long
    Dim lngNewLevel As Long
    Dim varMirror As Variant
    Dim lngDirection As Long
    Dim blnMirror As Boolean
    
    iMin = LBound(plngArray)
    iMax = UBound(plngArray)
    ReDim lngIndex((iMax - iMin + 3) \ 2)
    lngIndex(0) = iMin
    i = iMin
    ' Initial loop: locate cutoffs for each ordered section
    Do Until i >= iMax
        Select Case lngDirection
            Case 1
                Do Until i = iMax
                    Compare i, i + 1
                    If plngArray(i) > plngArray(i + 1) Then Exit Do
                    i = i + 1
                Loop
            Case -1
                Do Until i = iMax
                    Compare i, i + 1
                    If plngArray(i) < plngArray(i + 1) Then Exit Do
                    i = i + 1
                Loop
            Case Else
                Do Until i = iMax
                    Compare i, i + 1
                    If plngArray(i) <> plngArray(i + 1) Then Exit Do
                    i = i + 1
                Loop
                If i = iMax Then lngDirection = 1
        End Select
        If lngDirection = 0 Then
            Compare i, i + 1
            If plngArray(i) > plngArray(i + 1) Then
                lngDirection = -1
            Else
                lngDirection = 1
            End If
        Else
            lngLevel = lngLevel + 1
            lngIndex(lngLevel) = i * lngDirection
            lngDirection = 0
        End If
        i = i + 1
    Loop
    If Abs(lngIndex(lngLevel)) < iMax Then
        If lngDirection = 0 Then lngDirection = 1
        lngLevel = lngLevel + 1
        lngIndex(lngLevel) = i * lngDirection
    End If
    ' If the list is already sorted, exit
    If lngLevel <= 1 Then
        ' If sorted descending, reverse before exiting
        If lngIndex(lngLevel) < 0 Then
            For i = 0 To (iMax - iMin) \ 2
                Exchange iMin + i, iMax - i
            Next
        End If
        Erase lngIndex
        Exit Sub
    End If
    ' Main loop - merge section pairs together until only one section left
    ReDim varMirror(iMin To iMax)
    Do Until lngLevel = 1
        lngOldLevel = lngLevel
        For lngLevel = 1 To lngLevel - 1 Step 2
            If blnMirror Then
                SnakeSortMerge varMirror, lngIndex(lngLevel - 1), lngIndex(lngLevel), lngIndex(lngLevel + 1), plngArray
            Else
                SnakeSortMerge plngArray, lngIndex(lngLevel - 1), lngIndex(lngLevel), lngIndex(lngLevel + 1), varMirror
            End If
            lngNewLevel = lngNewLevel + 1
            lngIndex(lngNewLevel) = Abs(lngIndex(lngLevel + 1))
        Next
        If lngOldLevel Mod 2 = 1 Then
            If blnMirror Then
                For i = lngIndex(lngNewLevel) + 1 To iMax
                    SnakeMerge i, varMirror(i), lngIndex(lngNewLevel) + 1, iMax
                    plngArray(i) = varMirror(i)
                Next
            Else
                For i = lngIndex(lngNewLevel) + 1 To iMax
                    SnakeMerge i, plngArray(i), lngIndex(lngNewLevel) + 1, iMax
                    varMirror(i) = plngArray(i)
                Next
            End If
            lngNewLevel = lngNewLevel + 1
            lngIndex(lngNewLevel) = lngIndex(lngOldLevel)
        End If
        lngLevel = lngNewLevel
        lngNewLevel = 0
        blnMirror = Not blnMirror
    Loop
    ' Copy back to main array if necessary
    If blnMirror Then
        For i = iMin To iMax
            SnakeMerge i, varMirror(i), iMin, iMax
            plngArray(i) = varMirror(i)
        Next
    End If
End Sub

Private Sub SnakeSortMerge(pvarSource As Variant, plngLeft As Long, plngMid As Long, plngRight As Long, pvarDest As Variant)
    Dim L As Long
    Dim LMin As Long
    Dim LMax As Long
    Dim LStep As Long
    Dim R As Long
    Dim RMin As Long
    Dim RMax As Long
    Dim RStep As Long
    Dim O As Long
    
    If plngLeft <> 0 Then O = Abs(plngLeft) + 1
    If plngMid > 0 Then
        LMin = O
        LMax = Abs(plngMid)
        LStep = 1
    Else
        LMin = Abs(plngMid)
        LMax = O
        LStep = -1
    End If
    If plngRight > 0 Then
        RMin = Abs(plngMid) + 1
        RMax = Abs(plngRight)
        RStep = 1
    Else
        RMin = Abs(plngRight)
        RMax = Abs(plngMid) + 1
        RStep = -1
    End If
    L = LMin
    R = RMin
    Do
        Compare Abs(plngLeft), Abs(plngRight)
        If pvarSource(L) <= pvarSource(R) Then
            SnakeMerge O, pvarSource(L), Abs(plngLeft), Abs(plngRight)
            pvarDest(O) = pvarSource(L)
            If L = LMax Then
                For R = R To RMax Step RStep
                    O = O + 1
                    SnakeMerge O, pvarSource(R), Abs(plngLeft), Abs(plngRight)
                    pvarDest(O) = pvarSource(R)
                Next
                Exit Do
            End If
            L = L + LStep
        Else
            SnakeMerge O, pvarSource(R), Abs(plngLeft), Abs(plngRight)
            pvarDest(O) = pvarSource(R)
            If R = RMax Then
                For L = L To LMax Step LStep
                    O = O + 1
                    SnakeMerge O, pvarSource(L), Abs(plngLeft), Abs(plngRight)
                    pvarDest(O) = pvarSource(L)
                Next
                Exit Do
            End If
            R = R + RStep
        End If
        O = O + 1
    Loop
End Sub


' This code is not mine; I converted java code
' (included at end of this module) to VB6.
' It doesn't seem to work fully, but it's very close.
' I tossed in a GnomeSort at the end of it to
' ensure it ends up fully sorted, but that is
' clearly a temporary hack. If anyone can
' identify and correct the bug(s), please
' email me with the details: jdan37@hotmail.com
' Thanks much.
Public Sub SmoothSort(plngArray() As Long)
    Dim q As Long
    Dim R As Long
    Dim p As Long
    Dim b As Long
    Dim c As Long
    
    q = 1
    p = 1
    b = 1
    c = 1
    Do While q <> UBound(plngArray) + 1
        If p Mod 8 = 3 Then
            SmoothSift plngArray, R, b, c
            p = (p + 1) \ 4
            SmoothUp b, c
            SmoothUp b, c
        ElseIf p Mod 4 = 1 Then
            If q + c < UBound(plngArray) + 1 Then
                SmoothSift plngArray, R, b, c
            Else
                SmoothTrinkle plngArray, R, p, b, c
            End If
            Do
                SmoothDown b, c
                p = p * 2
            Loop While b <> 1
            p = p + 1
        End If
        q = q + 1
        R = R + 1
    Loop
    SmoothTrinkle plngArray, R, p, b, c
    Do While q <> 1
        q = q - 1
        If b = 1 Then
            R = R - 1
            p = p - 1
            Do While p Mod 2 = 0
                p = p / 2
                SmoothUp b, c
            Loop
        ElseIf b >= 3 Then
            p = p - 1
            R = R + c - b
            If p <> 0 Then SmoothSemiTrinkle plngArray, R, p, b, c
            SmoothDown b, c
            p = p * 2 + 1
            R = R + c
            SmoothSemiTrinkle plngArray, R, p, b, c
            SmoothDown b, c
            p = p * 2 + 1
        End If
    Loop
    GnomeSort plngArray ' Because I'm too lazy to debug the algorithm
End Sub

Private Sub SmoothUp(b As Long, c As Long)
    Dim temp As Long
    
    temp = b + c + 1
    c = b
    b = temp
End Sub

Private Sub SmoothDown(b As Long, c As Long)
    Dim temp As Long
    
    temp = b - c - 1
    b = c
    c = temp
End Sub

Private Sub SmoothSift(plngArray() As Long, ByVal R As Long, ByVal b As Long, ByVal c As Long)
    Dim r2 As Long
    
    Do While b >= 3
        r2 = R - b + c
        Compare r2, R - 1
        If plngArray(r2) < plngArray(R - 1) Then
            r2 = R - 1
            SmoothDown b, c
        End If
        Compare R, r2
        If plngArray(R) >= plngArray(r2) Then
            b = 1
        Else
            Exchange R, r2
            R = r2
            SmoothDown b, c
        End If
    Loop
End Sub

Private Sub SmoothTrinkle(plngArray() As Long, ByVal R As Long, ByVal p As Long, ByVal b As Long, ByVal c As Long)
    Dim r2 As Long
    Dim r3 As Long
    
    Do While p > 0
        Do While p Mod 2 = 0
            p = p \ 2
            SmoothUp b, c
        Loop
        r3 = R - b
        If p = 1 Then
            p = 0
        ElseIf plngArray(r3) <= plngArray(R) Then
            Compare r3, R
            p = 0
        Else
            Compare r3, R
            p = p - 1
            If b = 1 Then
                Exchange R, r3
                R = r3
            ElseIf b >= 3 Then
                r2 = R - b + c
                Compare r2, R - 1
                If plngArray(r2) < plngArray(R - 1) Then
                    r2 = R - 1
                    SmoothDown b, c
                    p = p * 2
                End If
                Compare r3, r2
                If plngArray(r3) >= plngArray(r2) Then
                    Exchange R, r3
                    R = r3
                Else
                    Exchange R, r2
                    SmoothDown b, c
                    p = 0
                End If
            End If
        End If
    Loop
    SmoothSift plngArray, R, b, c
End Sub

Private Sub SmoothSemiTrinkle(plngArray() As Long, ByVal R As Long, ByVal p As Long, ByVal b As Long, ByVal c As Long)
    Dim r1 As Long
    
    r1 = R - c
    Compare r1, R
    If plngArray(r1) > plngArray(R) Then
        Exchange R, r1
        SmoothTrinkle plngArray, r1, p, b, c
    End If
End Sub

'/*  An algorithm developed by Edsger Dijkstra based on the same principle as
' *  heapsort.  By using a postorder traversal of a heap-like structure, O(n)
' *  time complexity is achieved for already-sorted data.
' *  Time complexity:
' *      O(n log n) worst case
' *      O(n) best case
' *  Working memory:
' *      O(1) all cases
' */
'void smoothSort(uint[] data)
'out {
'    debug (100) printList(data);
'    checkSorted(data);
'}
'body {
'    void up(inout int b, inout int c) {
'        int temp = b + c + 1;
'        c = b;
'        b = temp;
'    }
'
'    void down(inout int b, inout int c) {
'        int temp = b - c - 1;
'        b = c;
'        c = temp;
'    }
'
'    void sift(int r, int b, int c) {
'        while (b >= 3) {
'            int r2 = r - b + c;
'
'            if (data[r2] < data[r - 1]) {
'                r2 = r - 1;
'                down(b, c);
'            }
'
'            if (data[r] >= data[r2]) {
'                b = 1;
'            } else {
'                swap(data[r], data[r2]);
'                r = r2;
'                down(b, c);
'            }
'        }
'    }
'
'    void trinkle(int r, int p, int b, int c) {
'        while (p > 0) {
'            int r3;
'
'            while (p % 2 == 0) {
'                p /= 2;
'                up(b, c);
'            }
'            r3 = r - b;
'
'            if (p == 1 || data[r3] <= data[r]) {
'                p = 0;
'            } else {
'                p--;
'                if (b == 1) {
'                    swap(data[r], data[r3]);
'                    r = r3;
'                } else if (b >= 3) {
'                    int r2 = r - b + c;
'                    if (data[r2] < data[r - 1]) {
'                        r2 = r - 1;
'                        down(b, c);
'                        p *= 2;
'                    }
'
'                    if (data[r3] >= data[r2]) {
'                        swap(data[r], data[r3]);
'                        r = r3;
'                    } else {
'                        swap(data[r], data[r2]);
'                        r = r2;
'                        down(b, c);
'                        p = 0;
'                    }
'                }
'            }
'        }
'        sift(r, b, c);
'    }
'
'    void semitrinkle(int r, int p, int b, int c) {
'        int r1 = r - c;
'        if (data[r1] > data[r]) {
'            swap(data[r], data[r1]);
'            trinkle(r1, p, b, c);
'        }
'    }
'
'    void swap(inout int x, inout int y) {
'        int temp = x;
'        x = y;
'        y = temp;
'    }
'
'    int q = 1, r, p = 1, b = 1, c = 1;
'
'    while (q != data.length) {
'        if (p % 8 == 3) {
'            sift(r, b, c);
'            p = (p+1)/4;
'            up(b, c); up(b, c);
'        } else if (p % 4 == 1) {
'            if (q + c < data.length) {
'                sift(r, b, c);
'            } else {
'                trinkle(r, p, b, c);
'            }
'            do {
'                down(b, c);
'                p *= 2;
'            } while (b != 1);
'            p++;
'        }
'        q++; r++;
'    }
'    trinkle(r, p, b, c);
'
'    while (q != 1) {
'        q--;
'        if (b == 1) {
'            r--; p--;
'            while (p % 2 == 0) {
'                p /= 2;
'                up(b, c);
'            }
'        } else if (b >= 3) {
'            p--;
'            r += c - b;
'            if (p != 0) semitrinkle(r, p, b, c);
'            down(b, c);
'            p = 2*p + 1;
'            r += c;
'            semitrinkle(r, p, b, c);
'            down(b, c);
'            p = 2*p + 1;
'        }
'    }
'}

