Attribute VB_Name = "basMain"
Option Explicit

Public Const CompanyName = "Ellisoft"
Public Const AppName = "Sort"

Public Const ParentKey = "Software\" & CompanyName
Public Const RegistryKey = ParentKey & "\" & AppName

Public Const BenchmarkExt = "bch"
Public Const BenchmarkDesc = "Benchmark settings"
Public Const ArrayExt = "arr"
Public Const ArrayDesc = "Array files"
Public Const TempFile = "\TimeTrial."

Public Const ErrorIgnore = 37001

Public Const Margin = 2 ' Horizontal / Vertical padding between labels, indicator, and interior
Public Const MaxColumns = 6
Public Const MaxRows = 4
Public Const MaxAreas = 18
Public Const BarLeft = 23 ' Relative left position of Bars

Public Enum AlgorithmEnum
    aeNone = 0
    aeQuicksort
    aeMedianThreeQuicksort
    aeShellsort
    aeCombsort
    aeShakersort
    aeJumpsort
    aeMergesort
    aeSnakesort
    aeSelectionsort
    aeInsertionsort
    aeGnomesort
    aeLinkedListSort
    aeHeapsort
    aeSmoothsort
    aeJSort
    aeShearsort
    aeCocktailsort
    aeBubblesort
End Enum

Public Enum OrderEnum
    oeRandom = 1
    oeAscending
    oeDescending
    oeCamel
    oePercent
    oeWeave
End Enum

Public Enum DataSourceEnum
    dseUserDefined = 0
    dseRandom = 1
End Enum

Public Enum DimensionEnum
    deSingle = 0
    deDynamic = 1
    deStatic = 2
End Enum

Public Enum KeyEnum
    keRandom = 0
    keUnique = 1
    keList = 2
End Enum

Public Enum VerifyEnum
    veNone = 0
    veFast
    veThorough
End Enum

Public Type ColumnType
    DataType As VbVarType
    Length As Long
End Type

Public Type BenchmarkType
    DataSource As DataSourceEnum
    Rows As Long
    Dimension As DimensionEnum
    Columns As Long
    Col() As ColumnType
    SortColumnIndex As Long
    KeyType As KeyEnum
    Keys As Long
    Order As OrderEnum
    Percent As Long
    Algorithm As Long ' Status flag (uses bitwise operators)
    Verify As VerifyEnum
    Arr As Variant
End Type

Public xp As clsWindowsXP
Public cfg As clsSettings
Public glngHeight As Long

Public Sub Main()
    Randomize
    Set xp = New clsWindowsXP
    Set cfg = New clsSettings
    frmMain.Show
End Sub

Public Sub CloseApp()
    Dim frm As Form
    
    Set cfg = Nothing
    Set xp = Nothing
    For Each frm In Forms
        Unload frm
        Set frm = Nothing
    Next
End Sub

Public Function GetSortName(penAlgorithm As AlgorithmEnum) As String
    Select Case penAlgorithm
        Case aeBubblesort: GetSortName = "Bubble sort"
        Case aeCocktailsort: GetSortName = "Cocktail sort"
        Case aeCombsort: GetSortName = "Comb sort"
        Case aeGnomesort: GetSortName = "Gnome sort"
        Case aeHeapsort: GetSortName = "Heap sort"
        Case aeInsertionsort: GetSortName = "Insertion sort"
        Case aeJSort: GetSortName = "JSort"
        Case aeJumpsort: GetSortName = "Jump sort"
        Case aeLinkedListSort: GetSortName = "Linked List sort"
        Case aeMedianThreeQuicksort: GetSortName = "Quicksort3"
        Case aeMergesort: GetSortName = "Merge sort"
        Case aeQuicksort: GetSortName = "Quick sort"
        Case aeSelectionsort: GetSortName = "Selection sort"
        Case aeShakersort: GetSortName = "Shaker sort"
        Case aeShearsort: GetSortName = "Shear sort"
        Case aeShellsort: GetSortName = "Shell sort"
        Case aeSmoothsort: GetSortName = "Smooth sort"
        Case aeSnakesort: GetSortName = "Snake sort"
        Case Else: GetSortName = ""
    End Select
End Function

Public Sub LoadArrayFromFile(pvarArray As Variant, pstrFile As String)
    Dim FileNumber As Integer

    FileNumber = FreeFile
    Open pstrFile For Binary As #FileNumber
    Get #FileNumber, , pvarArray
    Close #FileNumber
End Sub

Public Sub SaveArrayToFile(pvarArray As Variant, pstrFile As String)
    Dim FileNumber As Integer
    
    If xp.File.Exists(pstrFile) Then xp.File.Delete pstrFile
    FileNumber = FreeFile
    Open pstrFile For Binary As #FileNumber
    Put #FileNumber, , pvarArray
    Close #FileNumber
End Sub

Public Sub ToolbarIconSize(pfrm As Form)
    Dim i As Long
    Dim btn As MSComctlLib.Button
    
    With pfrm
        Set .tlb.ImageList = Nothing
        Set .tlb.DisabledImageList = Nothing
        Set .tlb.HotImageList = Nothing
        If cfg.LargeIcons Then
            Set .tlb.ImageList = .imglstLarge(0)
            Set .tlb.DisabledImageList = .imglstLarge(1)
            Set .tlb.HotImageList = .imglstLarge(2)
        Else
            Set .tlb.ImageList = .imglstSmall(0)
            Set .tlb.DisabledImageList = .imglstSmall(1)
            Set .tlb.HotImageList = .imglstSmall(2)
        End If
        For Each btn In .tlb.Buttons
            If btn.Style = tbrDefault Then btn.Image = btn.Key
        Next
        Set btn = Nothing
    End With
End Sub

Public Sub UpdateCodeFiles()
    Dim strCode As String
    
    
End Sub

Public Function VerifySort1(ByRef pvarArray As Variant) As Boolean
    Dim i As Long
    
    VerifySort1 = True
    For i = LBound(pvarArray) To UBound(pvarArray) - 1
        If pvarArray(i) > pvarArray(i + 1) Then
            VerifySort1 = False
            Exit For
        End If
    Next
End Function

